import torch
import torch.nn.functional as F
import matplotlib

import matplotlib.pyplot as plt
import numpy as np

x = torch.linspace(-np.pi, np.pi, 200)  # shape=(200)
x = torch.unsqueeze(x, dim=1)  # shape=(200, 1)
y = torch.sin(x) + 0.2 * torch.rand(x.size())  # shape=(200, 1))


class MyModule(torch.nn.Module):
    def __init__(self, n_feature, n_hidden, n_output):
        super(MyModule, self).__init__()
        self.hidden1 = torch.nn.Linear(n_feature, n_hidden)
        self.hidden2 = torch.nn.Linear(n_hidden, n_output)

    def forward(self, x):
        x = F.relu(self.hidden1(x))
        x = self.hidden2(x)
        return x


net = MyModule(n_feature=1, n_hidden=20, n_output=1)

optimizer = torch.optim.Adam(net.parameters(), lr=0.2)
loss_func = torch.nn.MSELoss()

from torch.jit import trace
traced_model = trace(net, y)
#
torch.jit.save(traced_model, "test")

# wykres:
plt.ion()
fig = plt.figure()
fig.canvas.draw()

for t in range(200):
    prediction = traced_model(x)
    loss = loss_func(prediction, y)

    optimizer.zero_grad()
    loss.backward()
    optimizer.step()

    if t % 5 == 0:  # wykres
        plt.cla()
        plt.ylim(-1.5, 1.5)
        plt.scatter(x.data.numpy(), y.data.numpy())
        plt.plot(x.data.numpy(), prediction.data.numpy(), 'r-', lw=5)
        plt.text(0.5, 0, 'Loss=%.4f' % loss.data.numpy(), fontdict={'size': 20, 'color': 'red'})
        plt.pause(0.1)
        fig.canvas.draw()

plt.ioff()
plt.show()
